#include <cstdio>
#include <cstdlib>
#include <vector>
#include <random>
#include <algorithm>
#include "goat.h"

static void ASSERT(bool p, const char* str)
{
  if(!p)
  {
    exit(1);
    fprintf(stderr, "%s\n", str);
  }
}

static std::vector<int> getcvx(std::vector<int> X, std::vector<int> Y)
{
  auto ccw = [&](int A, int B, int C)
  {
    return 1LL*(X[B]-X[A])*(Y[C]-Y[A])-1LL*(Y[B]-Y[A])*(X[C]-X[A]);
  };
  int N = X.size();
  std::vector<int> P;
  for(int i=0; i<N; ++i) P.push_back(i);
  std::sort(P.begin(), P.end(), [&](int a, int b){
    return std::make_pair(X[a], Y[a]) < std::make_pair(X[b], Y[b]);} );
  std::vector<int> U, L;
  for(auto x: P)
  {
    while(U.size() >= 2 && ccw(U[U.size()-2], U[U.size()-1], x) > 0)
      U.pop_back();
    U.push_back(x);
  }
  for(auto x: P)
  {
    while(L.size() >= 2 && ccw(L[L.size()-2], L[L.size()-1], x) < 0)
      L.pop_back();
    L.push_back(x);
  }
  
  if((int)L.size() == N) return L;
  L.pop_back(); std::reverse(U.begin(), U.end()); U.pop_back();
  for(auto x: U) L.push_back(x);
  return L;  
}

int main()
{
  int N; ASSERT(1==scanf("%d", &N), "invalid N");
  int Q; ASSERT(1==scanf("%d", &Q), "invalid Q");
  std::vector<int> X, Y;
  for(int i=0; i<N; ++i)
  {
    int xi, yi; ASSERT(2==scanf("%d%d", &xi, &yi), "invalid coordinate");
    X.push_back(xi); Y.push_back(yi);
  }
  
  int seed; ASSERT(1==scanf("%d", &seed), "invalid seed");
  
  std::vector<int> CV = getcvx(X, Y);
  
  std::mt19937 gen(seed);
  
  init(X, Y);
  long long accum1 = 0, accum2 = 0, accum3 = 0;
  for(int i=0; i<Q; ++i)
  {
    int a = gen()%CV.size();
    int b = gen()%(CV.size()-1); if(b >= a) ++b;
    int c = gen()%N;
    a = CV[a]; b = CV[b];
    while(c == a || c == b) c = gen()%N;
    long long ans = 0;
    if(gen()%3 == 0) ans = count(a, b, c);
    else if(gen()%3 == 1) ans = count(a, c, b);
    else ans = count(c, a, b);
    accum3 = (3001*accum3+ans)%1000000123;
    ans = ans*(i+1)*(i+2);
    accum1 ^= ans;
    accum2 = (accum2+ans)&( (1<<30)-1);
  }
  printf("%lld %lld %lld\n", accum1, accum2, accum3);
  return 0;
}


